#include <fcntl.h> // O_RDONLY ...
#include <stdio.h>
#include <gtk/gtk.h>
#include <string.h>
#include <stdint.h> //uint32_t
//#include <jpeglib.h>
#include <sys/mman.h>
#include <errno.h> //errno
#include <thread>
#include <string>
#include "blc.h"
#include "Display.hpp"

using namespace std;

class Client:public blc_array_tcp4_client{
    GtkWidget *window;
public:
    Client(string const &address, string const &port);
    void network_manager();
    unique_ptr<Display> display;
    unique_ptr<thread>  network_thread;
};

gboolean update_data(GtkWidget *widget, GdkFrameClock *frame_clock, void *user_data){
    auto client=reinterpret_cast<Client*>(user_data);
    client->recv_data();
    client->display->update();
    return G_SOURCE_CONTINUE;
}

Client::Client(string const &address, string const &port_name):blc_array_tcp4_client(address, port_name){
    string title(string(address)+":"+string(port_name));
    
    window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), title.c_str());
    
    display=make_unique<Display>(*this);
    gtk_container_add(GTK_CONTAINER(window), display->widget);
    gtk_widget_show_all(window);
    gtk_widget_add_tick_callback (display->widget, update_data, this, nullptr);

}

const char *channel_name, *fullscreen_option, *keyboard_mode, *address, *port_name;
GtkWidget *window;
GdkDisplay *main_display;
int interactive_mode=0;
GtkApplication *app;

blc_array mouse_array;
unique_ptr<Client> client;

char const *input_name;

int input_terminal;
int mode;

char const *format_string;
uint32_t format;

void ask_fullscreen(){
    gtk_window_fullscreen(GTK_WINDOW(window));
}

void ask_quit(){
    g_application_quit(G_APPLICATION(app));
    blc_quit();
}

void on_key_press(GtkWidget	*widget, GdkEventKey *event){
    char key;
    if (event->type == GDK_KEY_PRESS){
        key=event->keyval;
        fwrite(&key, 1, 1, stdout);
        fflush(stdout);
    }
}


void activate_cb(GApplication *app, gpointer){
    window=gtk_application_window_new(GTK_APPLICATION(app));
    gtk_window_set_title( GTK_WINDOW(window), "c_gtk_image");
    client=make_unique<Client>(address, port_name);
    if (client->display==nullptr) EXIT_ON_ARRAY_ERROR(client.get(), "Format not managed.");
    if (keyboard_mode) g_signal_connect(G_OBJECT(window), "key_press_event", G_CALLBACK (on_key_press), nullptr);
    gtk_widget_show_all(window);

}

/** Classical GTK application.
 * The first optional argument is the name of the experience. Otherwise all the existing shared memory are used.
 * */
int main(int argc, char *argv[]){
    int status=0;
    char const *g_debug, *min_str, *max_str;
    
    blc_program_set_description("Display the content of the blc_channel depending on its type on format");
    blc_program_add_option(&keyboard_mode, 'k', "keyboard", nullptr, "Send keyboard input to stdout", nullptr);
    blc_program_add_option(&min_str, 'm', "min", "FL32", "minimal value", nullptr);
    blc_program_add_option(&max_str, 'M', "max", "FL32", "maximal value", nullptr);
    blc_program_add_option(&fullscreen_option, 'F', "fullscreen", nullptr, "Set the window in fullscreen", nullptr);
    blc_program_add_option(&g_debug, 'g', "g-fatal-warnings", nullptr, "Debug gtk.", nullptr);
    blc_program_add_parameter(&address, "address", 0, "internet address", "localhost");
    blc_program_add_parameter(&port_name, "port", 0, "port name", "31440");
    blc_program_init(&argc, &argv, ask_quit);

    if (g_debug) {
        argc=2;
        argv=MANY_ALLOCATIONS(3, char*);
        argv[0]=strdup("Program");
        argv[1]=strdup("--g-fatal-warnings");
        argv[2]=nullptr;
    }
    gtk_disable_setlocale();
    gtk_init(&argc, &argv);
    app = gtk_application_new(nullptr, G_APPLICATION_FLAGS_NONE);
    g_signal_connect (app, "activate", G_CALLBACK (activate_cb), nullptr);
    status = g_application_run(G_APPLICATION(app), 0, nullptr);
    g_object_unref(app);
    return status;
}

