#include "UIN8_UYVY.hpp"

UIN8_UYVY_display::UIN8_UYVY_display(blc_array const &array):Display(array){
    int Y, Cb, Cr;
    int i, j;
    uint8_t  G_tmp, B;
    uint8_t  R[256];
    
    width=dims[1].length;
    height=dims[2].length;
    image_surface=cairo_image_surface_create( CAIRO_FORMAT_RGB24, width, height);
    
    if (RGBA_from_YUYV==nullptr){
        RGBA_from_YUYV= MANY_ALLOCATIONS(256*256*256, uint32_t);
        FOR(Y, 256){
            FOR(j,256) R[j]= CLIP_UCHAR(Y+1.13983*(j-128)); //It does not depend on Cb
            FOR(Cb, 256)
            {
                B =  CLIP_UCHAR(Y+2.03211*(Cb-128)); // It does not depend on Cr
                G_tmp = - 0.58060*(Cb-128);
                
                FOR(Cr, 256){
                    i = Y + (Cb << 8) + (Cr << 16);
                    // Version Wikipedia
                    RGBA_from_YUYV[i] = (R[Cr]<<16) + (((uchar)CLIP_UCHAR(Y-0.39465*(Cr-128) + G_tmp))<< 8)+ B + (255<<24);
                }
            }
        }
    }
}

void UIN8_UYVY_display::update(){
    int Y, U = 128, V = 128, j;
    //uints32 is the array data, surface_data is the data of the image
    uint32_t* tmp_pixels=reinterpret_cast<uint32_t*>(image_data);
    for (int i=0; i!=size;){
           U = uchars[i++];
           Y = uchars[i++];
           j = Y + (U << 8) + (V << 16);
           tmp_pixels[0]=RGBA_from_YUYV[j];
    
           V = uchars[i++];
           Y = uchars[i++];
           j = Y + (U << 8) + (V << 16);
           tmp_pixels[1]=RGBA_from_YUYV[j];
           tmp_pixels+=2;
    }
}


void UIN8_UYVY_display::snprint_pixel(char *text, int size_max, int i, int j){
    uint8_t Y, U, V;
    Y=uchars[i*2+j*width*2+1];
    U=uchars[i*4+j*width*2];
    V=uchars[i*4+j*width*2+2];
    snprintf(text, size_max, "%d,%d: Y:%d, U:%d, V:%d", i, j, Y, U, V);
}

void UIN8_UYVY_display::snprint_histogram(char *text, int size_max, int i){
    uint8_t Y, U, V;
}


void UIN8_UYVY_display::show_histogram(){
    

}

void UIN8_UYVY_display::hide_histogram(){

}

uint32_t *UIN8_UYVY_display::RGBA_from_YUYV = nullptr;


void UIN8_UYVY_histogram::snprint(char *text, int size_max, int i, int j){
    uint8_t Y, U, V;
    Y=array->uchars[i*2+j*width*2+1];
    U=array->uchars[i*4+j*width*2];
    V=array->uchars[i*4+j*width*2+2];
    SPRINTF(text, "Y[%d]=%f.2%%", x, histogram[x]*100/(float)array->size);
}
