#include "UIN8_Y800.hpp"
#include <blc_image.h>

using namespace std;

#define Y800_to_RGBA(val) (val+(val<<8)+(val<<16)+(255<<24))

UIN8_Y800_image::UIN8_Y800_image(blc_array const &array):Image(array){
    width=dims[0].length;
    height=dims[1].length;
    surface=cairo_image_surface_create(CAIRO_FORMAT_RGB24, width, height);
    pixels=reinterpret_cast<uint32_t*>(cairo_image_surface_get_data(surface));
    gtk_image_set_from_surface(GTK_IMAGE(image), surface);
    
    color_map.resize(256);
    for (int i=0; i!=color_map.size(); i++) color_map[i]=Y800_to_RGBA(i);
    
    //   false_colors_button=blgtk_add_toggle_tool_button(toolbar, "false\ncolors", "applications-graphics", G_CALLBACK(false_color_cb), nullptr);
}

void UIN8_Y800_image::update_pointer_info(){
    int i, j;
    if (get_pointer_position(i, j)){
        pointer_info=to_string(i)+","+to_string(j)+" Y:"+to_string(uchars[i+j*width]);
    }
}

void UIN8_Y800_image::update(){
    uint8_t grey;

    for (int i=0; i<total_length; i++) {
        grey = uchars[i];
        pixels[i]=color_map[grey];
    }
    gtk_widget_queue_draw(image);
}





// *** Histogram ***

UIN8_Y800_histogram::UIN8_Y800_histogram(blc_array const &array):Histogram(array){
    surface=cairo_image_surface_create(CAIRO_FORMAT_RGB24, 256, 256);
    pixels=reinterpret_cast<uint32_t*>(cairo_image_surface_get_data(surface));
}

void UIN8_Y800_histogram::update(){
    int i, j;
    uint8_t grey, hist_max;
    FOR(i, 256) {
        grey=uchars[i];
        histogram[grey]++;
    }
    hist_max = *max_element(uchars, uchars+size);
    FOR(i, 256){
        FOR(j, histogram[i]*255/hist_max) pixels[(255-j)*256+i]=color_map[i];
        pixels[(255-histogram[i]*255/hist_max)*256+i]=255<<24;
        FOR(j, 255-histogram[i]*255/hist_max) pixels[j*256+i]=0;
    }
    gtk_widget_queue_draw(image);
}

void UIN8_Y800_histogram::update_pointer_info(){
    int i;
    
    if (get_pointer_position(i)){
        pointer_info="Y["+to_string(i)+"]="+to_string(histogram[i]*100/static_cast<float>(size))+"%";
    }
}
